function find_in_equations(model,object,output)

// PURPOSE: in a model, finds the equations where an object
// (endogenous, exogenous, residual, coefficient) is used
// ------------------------------------------------------------
// INPUT:
// * model = a model typed list created by the function
//   create_model
// * object = the name of the object to search
// * output = the place to save the results (optional; default:
//   the working space)
// ------------------------------------------------------------
// OUTPUT:
// nothing: the result is displayed on the screen
// ------------------------------------------------------------
// Copyright: Eric Dubois 2014-2015
// http://dubois.ensae.net/grocer.html
   

[nargout,nargin]=argn(0)
if nargin >3 then
    output=%io(2)
end

equations= model('equations')
name_equations= model('name eq')
endogenous= model('name endo')
ind_endogenous=find(endogenous == object)
if ~isempty(ind_endogenous) then
   listeqperendo=find(full(model('eq endos')(ind_endogenous,:)) == 1)
   disp('endogenous '+object+' found in equation(s):')
   for i=1:size(listeqperendo,'*')
      ind_i=listeqperendo(i)
      disp(name_equations(ind_i)+': '+equations(ind_i))
      disp(' ')
   end

else
   exogenous= model('name exo')
   ind_exogenous=find(exogenous == object)
   if ~isempty(ind_exogenous) then
      listeqperexo=find(full(model('eq exos')(ind_exogenous,:)) == 1)
      disp('exogenous '+object+' found in equation(s):')
      for i=1:size(listeqperexo,'*')
         ind_i=listeqperexo(i)
         disp(name_equations(ind_i)+': '+equations(ind_i))
         disp(' ')
      end
   else 
      residuals= model('name resid')
      ind_resid=find(residuals == object)
      if ~isempty(ind_resid) then
         listeqperresid=find(full(model('eq resids')(ind_resid,:)) == 1)
         disp('residual '+object+' found in equation(s):')
         for i=1:size(listeqperresid,'*')
            ind_i=listeqperresid(i)
            disp(name_equations(ind_i)+': '+equations(ind_i))
            disp(' ')
         end
      else 
         coeffs= model('name coeff')
         ind_coeffs=find(coeffs == object)
         if ~isempty(ind_coeffs) then
            listeqpercoeff=find(full(model('eq coeffs')(ind_coeffs,:)) == 1)
            disp('coefficient '+object+' found in equation(s):')
            disp(' ')
            for i=1:size(listeqpercoeff,'*')
               ind_i=listeqpercoeff(i)
               disp(name_equations(ind_i)+': '+equations(ind_i))
            end
         else 
            disp('variable '+object+' not found in the model equations')
         end
      end
   end
end

endfunction
