function tsmat=tsmat_extrap(tsmat,varargin)
 
// PURPOSE: exatrpolates variables of a tsmat
// ------------------------------------------------------------
// INPUT:
// * tsmat = a tsmat
// * varargin = N sequences variable,bounds,type_extrap,values
//   with:
//   - variable = a string, the names of a variable in the
//     tsmat that will be extrapolated
//   - bounds = the bounds over which the extrapolation is
//     made
//   - type_extrap = a string, the way the extrapolation will 
//     be made ('level' if it is done by entering its values ;
//     'growthr rate' if its is done by applying growth rate to
//     the previous value and 'delta rate' if it is done by 
//     adding values to the previous one)
//   - values = either a scalar or a vector of the size of the 
//     bounds, providing the value(s) used for extrapolation
// ------------------------------------------------------------
// OUTPUT:
// * tsmat = the original tsmat with the given variables
//   extrapolated
// ------------------------------------------------------------
// Copyright: Eric Dubois 2018-2019
// http://dubois.ensae.net/grocer.html 
    
nargin=length(varargin)
tsmat_dates=tsmat('dates')    
tsmat_series=tsmat('series') 
tsmat_names=tsmat('names') 
[nobs,nsers]=size(tsmat_series)   
tsmat_freq=tsmat('freq')  

datenum_$0=tsmat_dates($)
datenum_$=datenum_$0
for i=2:4:nargin-2
    dates_i=varargin(i)
    [datnum_i1,fq_i1]=date2num_fq(dates_i(1))
    [datnum_i2,fq_i2]=date2num_fq(dates_i(2))
    if cumprod(fq_i1) ~= cumprod(tsmat_freq) | fq_i1(1) ~= tsmat_freq then
       error(dates_i(1)+' has not the same frequency ('+string(fq_i1)+') as the tsmat ('+ string(tsmat_freq)+')')
    end
    if fq_i2 ~= tsmat_freq then
       error(dates_i(1)+' has not the same frequency ('+string(fq_i2)+') as the tsmat ('+ string(tsmat_freq)+')')
    end
    if datnum_i1 > datenum_$0+1 then
       error(dates_i(1)+' is more than one period later than the last date of the tsmat')
    end
    datenum_$=max(datenum_$,datnum_i2)
end
tsmat_series=[tsmat_series ; %nan*ones(datenum_$-tsmat_dates($),nsers)]
tsmat_dates=[tsmat_dates(1):datenum_$]'

for i=1:4:nargin-2  
   name_i=varargin(i)
   dates_i=varargin(i+1)
   proj_i=varargin(i+3)

   ind_i=find(tsmat_names == name_i)
   ind_dat1=date2num(dates_i(1))-tsmat_dates(1)+1
   ind_dat2=date2num(dates_i(2))-tsmat_dates(1)+1
   ndates=ind_dat2-ind_dat1+1
   
   nvals=size(proj_i,'*')
   if nvals == 1 then
      proj_i=proj_i*ones(ndates,1)
   end
   
   select varargin(i+2)
       
   case 'level' then
       tsmat_series(ind_dat1:ind_dat2,ind_i)=proj_i
       
   case 'growth rate' then
       tsmat_series(ind_dat1:ind_dat2,ind_i)=cumprod(1+proj_i)*tsmat_series(ind_dat1-1,ind_i)
       
   case 'delta rate' then
       tsmat_series(ind_dat1:ind_dat2,ind_i)=cumsum( proj_i)+tsmat_series(ind_dat1-1,ind_i)
       
   else 
      error('not an available option:' +string(varargin(i+2)))
   end
end

tsmat('series')=tsmat_series   
tsmat('dates')=tsmat_dates


endfunction
